"""
Color management system for custom UI components
"""

from .color_palette import GLOBAL_COLOR_PALETTE, create_background_style

__all__ = [
    "GLOBAL_COLOR_PALETTE",
    "create_background_style",
]
